@@ -30,11 +30,10 @@
 #include <string.h>
 #include <time.h>
 
-#include "favicon.h"
-#include "support.h"
-#include "feed.h"
 #include "common.h"
 #include "debug.h"
+#include "favicon.h"
+#include "feed.h"
 #include "html.h"
 
 typedef struct faviconDownloadCtxt {
@@ -76,21 +75,15 @@ GdkPixbuf * favicon_load_from_cache(const gchar *id) {
 	return result;
 }
 
-gboolean favicon_update_needed(const gchar *id, updateStatePtr updateState, GTimeVal *now) {
-	struct stat	statinfo;
-	gchar		*filename;
+gboolean
+favicon_update_needed(const gchar *id, updateStatePtr updateState, GTimeVal *now)
+{
 	gboolean	result = FALSE;
-	
-	/* try to load a saved favicon */
-	filename = common_create_cache_filename("cache" G_DIR_SEPARATOR_S "favicons", id, "png");
-	
-	if(0 == stat((const char*)filename, &statinfo)) {
-		/* check creation date and update favicon if older than one month */
-		if(now->tv_sec > (updateState->lastFaviconPoll.tv_sec + 60*60*24*31))
-			result = TRUE;
-	}
-	g_free(filename);
-	
+
+	/* check creation date and update favicon if older than one month */
+	if (now->tv_sec > (updateState->lastFaviconPoll.tv_sec + 60*60*24*31))
+		result = TRUE;
+
 	return result;
 }
 
@@ -110,118 +103,123 @@ void favicon_remove_from_cache(const gchar *id) {
 	debug_exit("favicon_remove");
 }
 
-static void favicon_download_icon_cb(requestPtr request) {
-	faviconDownloadCtxtPtr	ctxt = (faviconDownloadCtxtPtr)request->user_data;
+static void
+favicon_download_icon_cb (const struct updateResult * const result, gpointer user_data, updateFlags flags)
+{
+	faviconDownloadCtxtPtr	ctxt = (faviconDownloadCtxtPtr)user_data;
 	gchar		*tmp;
 	GError		*err = NULL;
 	gboolean	success = FALSE;
 	
-	debug4(DEBUG_UPDATE, "icon download processing (%s, %d bytes, content type %s) for favicon %s", request->source, request->size, request->contentType, ctxt->id);
+	debug4 (DEBUG_UPDATE, "icon download processing (%s, %d bytes, content type %s) for favicon %s", result->source, result->size, result->contentType, ctxt->id);
 
-	if(request->data && 
-	   request->size > 0 && 
-	   request->contentType /*&&*/
+	if (result->data && 
+	    result->size > 0 && 
+	    result->contentType /*&&*/
 	   /* the MIME type is wrong much too often, so we cannot check it... */
-	   /*(!strncmp("image", request->contentType, 5))*/) {
-		GdkPixbufLoader *loader = gdk_pixbuf_loader_new();
+	   /*(!strncmp("image", result->contentType, 5))*/) {
+		GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();
 		GdkPixbuf *pixbuf;
-		if(gdk_pixbuf_loader_write(loader, (guchar *)request->data, request->size, &err)) {
-			if(gdk_pixbuf_loader_close(loader, &err)) {
-				pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
-				if(pixbuf) {
-					tmp = common_create_cache_filename("cache" G_DIR_SEPARATOR_S "favicons", ctxt->id, "png");
-					debug2(DEBUG_UPDATE, "saving favicon %s to file %s", ctxt->id, tmp);
-					if(!gdk_pixbuf_save(pixbuf, tmp, "png", &err, NULL)) {
-						g_warning("Could not save favicon (id=%s) to file %s!", ctxt->id, tmp);
+		if (gdk_pixbuf_loader_write (loader, (guchar *)result->data, (gsize)result->size, &err)) {
+			if (gdk_pixbuf_loader_close (loader, &err)) {
+				pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+				if (pixbuf) {
+					tmp = common_create_cache_filename ("cache" G_DIR_SEPARATOR_S "favicons", ctxt->id, "png");
+					debug2 (DEBUG_UPDATE, "saving favicon %s to file %s", ctxt->id, tmp);
+					if (!gdk_pixbuf_save (pixbuf, tmp, "png", &err, NULL)) {
+						g_warning ("Could not save favicon (id=%s) to file %s!", ctxt->id, tmp);
 					} else {
 						success = TRUE;
 						/* Run favicon-updated callback */
-						if(ctxt->callback)
-							(ctxt->callback)(ctxt->user_data);
+						if (ctxt->callback)
+							(ctxt->callback) (ctxt->user_data);
 					}
-					g_free(tmp);
+					g_free (tmp);
 				} else {
-					debug0(DEBUG_UPDATE, "gdk_pixbuf_loader_get_pixbuf() failed!");
+					debug0 (DEBUG_UPDATE, "gdk_pixbuf_loader_get_pixbuf() failed!");
 				}
 			} else {
-				debug0(DEBUG_UPDATE, "gdk_pixbuf_loader_close() failed!");
+				debug0 (DEBUG_UPDATE, "gdk_pixbuf_loader_close() failed!");
 			}
 		} else {
-			debug0(DEBUG_UPDATE, "gdk_pixbuf_loader_write() failed!");
-			gdk_pixbuf_loader_close(loader, NULL);
+			debug0 (DEBUG_UPDATE, "gdk_pixbuf_loader_write() failed!");
+			gdk_pixbuf_loader_close (loader, NULL);
 		}
 
-		if(err) {
-			g_warning("%s\n", err->message);
-			g_error_free(err);
+		if (err) {
+			g_warning ("%s\n", err->message);
+			g_error_free (err);
 		}
 
-		g_object_unref(loader);
+		g_object_unref (loader);
 	} else {
-		debug1(DEBUG_UPDATE, "No data in download result for favicon %s!", ctxt->id);
+		debug1 (DEBUG_UPDATE, "No data in download result for favicon %s!", ctxt->id);
 	}
 	
-	update_request_free(request);
-	
-	if(!success) {
-		favicon_download_run(ctxt);	/* try next... */
+	if (!success) {
+		favicon_download_run (ctxt);	/* try next... */
 	} else {
-		g_slist_free(ctxt->urls);
-		g_free(ctxt);
+		g_slist_free (ctxt->urls);
+		g_free (ctxt);
 	}
 }
 
-static void favicon_download_html_cb(requestPtr request) {
-	faviconDownloadCtxtPtr	ctxt = (faviconDownloadCtxtPtr)request->user_data;
+static void
+favicon_download_html_cb (const struct updateResult * const result, gpointer user_data, updateFlags flags) {
+	faviconDownloadCtxtPtr	ctxt = (faviconDownloadCtxtPtr)user_data;
 	
-	if(request->size > 0 && request->data) {
-		gchar *iconUri = html_discover_favicon(request->data, request->source);
-		if(iconUri) {
-			debug2(DEBUG_UPDATE, "found link for favicon %s: %s", ctxt->id, iconUri);
-			update_request_free(request);
-			request = update_request_new(ctxt->user_data);
+	if (result->size > 0 && result->data) {
+		gchar *iconUri = html_discover_favicon (result->data, result->source);
+		if (iconUri) {
+			updateRequestPtr request;
+			
+			debug2 (DEBUG_UPDATE, "found link for favicon %s: %s", ctxt->id, iconUri);
+			request = update_request_new ();
 			request->source = iconUri;
-			request->options = ctxt->options;
-			request->callback = &favicon_download_icon_cb;
-			request->user_data = ctxt;
-			update_execute_request(request);
+			request->options = update_options_copy (ctxt->options);
+			update_execute_request (ctxt->user_data, request, favicon_download_icon_cb, ctxt, flags);
+			
 			return;
 		}
 	}
-	debug1(DEBUG_UPDATE, "No link for favicon %s found!", ctxt->id);
+	debug1 (DEBUG_UPDATE, "No link for favicon %s found!", ctxt->id);
 
-	update_request_free(request);
-	favicon_download_run(ctxt);	/* no sucess, try next... */
+	favicon_download_run (ctxt);	/* no sucess, try next... */
 }
 
-static void favicon_download_run(faviconDownloadCtxtPtr ctxt) {
-	gchar		*url;
-	struct request	*request;
+static void
+favicon_download_run (faviconDownloadCtxtPtr ctxt)
+{
+	gchar			*url;
+	updateRequestPtr	request;
+	update_result_cb	callback;
 
 	debug_enter("favicon_download_run");
 	
-	if(ctxt->urls) {
+	if (ctxt->urls) {
 		url = (gchar *)ctxt->urls->data;
-		ctxt->urls = g_slist_remove(ctxt->urls, url);
-		debug2(DEBUG_UPDATE, "favicon %s trying URL: %s\n", ctxt->id, url);
+		ctxt->urls = g_slist_remove (ctxt->urls, url);
+		debug2 (DEBUG_UPDATE, "favicon %s trying URL: %s", ctxt->id, url);
 
-		request = update_request_new(ctxt->user_data);
+		request = update_request_new ();
 		request->source = url;
-		request->options = ctxt->options;
-		request->user_data = ctxt;
+		request->options = update_options_copy (ctxt->options);
 
-		if(strstr(url, "/favicon.ico"))
-			request->callback = &favicon_download_icon_cb;	
+		if (strstr (url, "/favicon.ico"))
+			callback = favicon_download_icon_cb;	
 		else
-			request->callback = &favicon_download_html_cb;	
+			callback = favicon_download_html_cb;
 
-		update_execute_request(request);
+		update_execute_request (ctxt->user_data, request, callback, ctxt, 0);
 	} else {
-		debug1(DEBUG_UPDATE, "favicon %s could not be downloaded!\n", ctxt->id);
-		g_free(ctxt);
+		debug1 (DEBUG_UPDATE, "favicon %s could not be downloaded!", ctxt->id);
+		/* Run favicon-updated callback */
+		if (ctxt->callback)
+			(ctxt->callback) (ctxt->user_data);
+		g_free (ctxt);
 	}
 	
-	debug_exit("favicon_download_run");
+	debug_exit ("favicon_download_run");
 }
 
 static gint count_slashes(const gchar *str) {
@@ -229,7 +227,7 @@ static gint count_slashes(const gchar *str) {
 	gint		slashes = 0;
 	
 	slashes = 0;
-	while(*tmp) {if(*tmp == '/') slashes++;tmp++; }
+	while(*tmp) { if(*tmp == '/') slashes++;tmp++; }
 	
 	return slashes;
 }
@@ -262,23 +260,25 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 	/* In the following These URLs will be prepared here and passed as a list to 
 	   the download function that will then process in order
 	   until success or the end of the list is reached... */
-	debug1(DEBUG_UPDATE, "preparing download URLs for favicon %s...\n", ctxt->id);
+	debug1(DEBUG_UPDATE, "preparing download URLs for favicon %s...", ctxt->id);
 	
 	/* case 1. */
 	if(html_url) {
 		tmp = g_strdup(html_url);	
 		ctxt->urls = g_slist_append(ctxt->urls, tmp);
-		debug1(DEBUG_UPDATE, "(1) adding favicon search URL: %s\n", tmp);
+		debug1(DEBUG_UPDATE, "(1) adding favicon search URL: %s", tmp);
 	}
 
 	/* case 2. */
 	g_assert(source_url);	
-	tmp = tmp2 = g_strdup(source_url);
-	tmp = strrchr(tmp, '/');
-	if(tmp) {
-		*tmp = 0;
-		ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-		debug1(DEBUG_UPDATE, "(2) adding favicon search URL: %s\n", tmp2);
+	if(*source_url != '|') {
+		tmp = tmp2 = g_strdup(source_url);
+		tmp = strrchr(tmp, '/');
+		if(tmp) {
+			*tmp = 0;
+			ctxt->urls = g_slist_append(ctxt->urls, tmp2);
+			debug1(DEBUG_UPDATE, "(2) adding favicon search URL: %s", tmp2);
+		}
 	}
 	
 	/* case 3. */
@@ -293,14 +293,14 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 					tmp = tmp2;
 					tmp2 = g_strdup_printf("%s/favicon.ico", tmp);
 					ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-					debug1(DEBUG_UPDATE, "(3) adding favicon source URL: %s\n", tmp2);
+					debug1(DEBUG_UPDATE, "(3) adding favicon source URL: %s", tmp2);
 				}
 			}
 			g_free(tmp);
 		}
 	}
 	
-	if(2 < count_slashes(source_url)) {
+	if(*source_url != '|' && 2 < count_slashes(source_url)) {
 		/* case 4 */
 		tmp = tmp2 = g_strdup(source_url);
 		tmp = strrchr(tmp, '/');
@@ -309,7 +309,7 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 			tmp = tmp2;
 			tmp2 = g_strdup_printf("%s/favicon.ico", tmp);
 			ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-			debug1(DEBUG_UPDATE, "(4) adding favicon source URL: %s\n", tmp2);
+			debug1(DEBUG_UPDATE, "(4) adding favicon source URL: %s", tmp2);
 		}
 		g_free(tmp);
 	
@@ -323,7 +323,7 @@ void favicon_download(const gchar *id, const gchar *html_url, const gchar *sourc
 				tmp = tmp2;
 				tmp2 = g_strdup_printf("%s/favicon.ico", tmp);
 				ctxt->urls = g_slist_append(ctxt->urls, tmp2);
-				debug1(DEBUG_UPDATE, "(5) adding favicon source URL: %s\n", tmp2);
+				debug1(DEBUG_UPDATE, "(5) adding favicon source URL: %s", tmp2);
 			}
 		}
 		g_free(tmp);
