@@ -2,7 +2,7 @@
  * @file html.c HTML file handling / feed auto discovery
  * 
  * Copyright (C) 2004 ahmed el-helw <ahmedre@cc.gatech.edu>
- * Copyright (C) 2004-2006 Lars Lindner <lars.lindner@gmx.net>
+ * Copyright (C) 2004-2007 Lars Lindner <lars.lindner@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +28,12 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
-#include "support.h"
-#include "callbacks.h"
+
+#include "common.h"
 #include "debug.h"
 #include "html.h"
+#include "xml.h"
+#include "ui/ui_mainwindow.h"
 
 enum {
 	LINK_FAVICON,
@@ -46,7 +48,7 @@ static gchar *checkLinkRef(const gchar* str, gint linkType) {
 	gchar	quote; 
 	
 	/*debug1(DEBUG_PARSING, "checking link %s", str); */
-	tmp = liferea_strcasestr(str, "href=");
+	tmp = common_strcasestr(str, "href=");
 	if(NULL == tmp) return NULL;
 	tmp += 5;
 	/* skip spaces up to the first quote. This is really slightly
@@ -69,23 +71,25 @@ static gchar *checkLinkRef(const gchar* str, gint linkType) {
 
 	res = g_strndup(tmp, len);
 
-	if (linkType == LINK_FAVICON) {
-		if((liferea_strcasestr(str, "shortcut icon")!=NULL ||
-		    liferea_strcasestr(str, "icon")!=NULL) &&
-		   ((liferea_strcasestr(str, "image/x-icon")!=NULL) ||
-		    (liferea_strcasestr(str, "image/png")!=NULL) ||
-		    (liferea_strcasestr(str, "image/gif")!=NULL)))
+	if(linkType == LINK_FAVICON) {
+		if(((NULL != common_strcasestr(str, "shortcut icon")) ||
+		    (NULL != common_strcasestr(str, "icon"))) &&
+		   ((NULL != common_strcasestr(str, "image/x-icon")) ||
+		    (NULL != common_strcasestr(str, "image/png")) ||
+		    (NULL != common_strcasestr(str, "image/gif"))))
 			return res;
-	} else if (linkType == LINK_RSS_ALTERNATE){
-		if((liferea_strcasestr(str, "alternate")!=NULL) &&
-		   ((liferea_strcasestr(str, "text/xml")!=NULL) || 
-		    (liferea_strcasestr(str, "rss+xml")!=NULL) ||
-		    (liferea_strcasestr(str, "rdf+xml")!=NULL) ||
-		    (liferea_strcasestr(str, "atom+xml")!=NULL)))
+	} else if(linkType == LINK_RSS_ALTERNATE){
+		if((common_strcasestr(str, "alternate")!=NULL) &&
+		   ((common_strcasestr(str, "text/xml")!=NULL) || 
+		    (common_strcasestr(str, "rss+xml")!=NULL) ||
+		    (common_strcasestr(str, "rdf+xml")!=NULL) ||
+		    (common_strcasestr(str, "atom+xml")!=NULL)))
 			return res;
-	} else if (linkType == LINK_NORMAL_ANCHOR){
-		if((strstr(res, "rdf")) || (strstr(res, "xml")) ||
-		   (strstr(res, "rss")))
+	} else if(linkType == LINK_NORMAL_ANCHOR){
+		if((strstr(res, "rdf")) || 
+		   (strstr(res, "xml")) ||
+		   (strstr(res, "rss")) ||
+		   (strstr(res, "atom")))
 			return res;
 	}
 	g_free(res);
@@ -101,7 +105,7 @@ static gchar *search_links(const gchar* data, gint linkType) {
 	gchar	*endptr;
 	
 	while(1) {
-		ptr = liferea_strcasestr(tmp, ((linkType != LINK_NORMAL_ANCHOR)? "<link " : "<a "));
+		ptr = common_strcasestr(tmp, ((linkType != LINK_NORMAL_ANCHOR)? "<link " : "<a "));
 		if(NULL == ptr)
 			break;
 		
