@@ -105,7 +105,6 @@ static void parseChannel(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 		else if(!xmlStrcmp(cur->name, BAD_CAST"ttl")) {
  			if(NULL != (tmp = common_utf8_fix(xmlNodeListGetString(ctxt->doc, cur->xmlChildrenNode, TRUE)))) {
 				subscription_set_default_update_interval(ctxt->subscription, atoi(tmp));
-				subscription_set_update_interval(ctxt->subscription, atoi(tmp));
 				g_free(tmp);
 			}
 		}
@@ -122,10 +121,11 @@ static void parseChannel(feedParserCtxtPtr ctxt, xmlNodePtr cur) {
 				g_free(tmp);
 			}
 		}
-		else if(!xmlStrcmp(cur->name, BAD_CAST"description")) {
- 			if(NULL != (tmp = common_utf8_fix(xhtml_extract (cur, 0, NULL)))) {
-				feed_set_description(ctxt->feed, tmp);
-				g_free(tmp);
+		else if (!xmlStrcmp (cur->name, BAD_CAST"description")) {
+ 			tmp = common_utf8_fix (xhtml_extract (cur, 0, NULL));
+			if (tmp) {
+				metadata_list_set (&ctxt->subscription->metadata, "description", tmp);
+				g_free (tmp);
 			}
 		}
 		
