@@ -32,7 +32,9 @@
 #include "ui/ui_mainwindow.h"
 #include "ui/ui_node.h"
 
+/* The allowed feed protocol prefixes (see http://25hoursaday.com/draft-obasanjo-feed-URI-scheme-02.html) */
 #define FEED_PROTOCOL_PREFIX "feed://"
+#define FEED_PROTOCOL_PREFIX2 "feed:"
 
 subscriptionPtr
 subscription_new (const gchar *source,
@@ -52,15 +54,35 @@ subscription_new (const gchar *source,
 	subscription->defaultInterval = -1;
 	
 	if (source) {
-		gchar *tmp, *uri = g_strdup (source);
+		gboolean feedPrefix = FALSE;
+		gchar *uri = g_strdup (source);
 		g_strstrip (uri);	/* strip confusing whitespaces */
 		
-		/* strip feed protocol prefix */
-		tmp = uri;
-		if (tmp == strstr (tmp, FEED_PROTOCOL_PREFIX))
-			tmp += strlen (FEED_PROTOCOL_PREFIX);
+		/* strip feed protocol prefix variant 1 */
+		if (uri == strstr (uri, FEED_PROTOCOL_PREFIX)) {
+			gchar *tmp = uri;
+			uri = g_strdup (uri + strlen (FEED_PROTOCOL_PREFIX));
+			g_free (tmp);
+			feedPrefix = TRUE;
+		}
+
+		/* strip feed protocol prefix variant 2 */
+		if (uri == strstr (uri, FEED_PROTOCOL_PREFIX2)) {
+			gchar *tmp = uri;
+			uri = g_strdup (uri + strlen (FEED_PROTOCOL_PREFIX2));
+			g_free (tmp);
+			feedPrefix = TRUE;
+		}
 			
-		subscription_set_source (subscription, tmp);
+		/* ensure protocol prefix (but only for feed:[//] URIs to avoid 
+		   breaking local file and command line subscriptions) */
+		if (feedPrefix && !strstr (uri, "://")) {
+			gchar *tmp = uri;
+			uri = g_strdup_printf ("http://%s", uri);
+			g_free (tmp);
+		}
+			
+		subscription_set_source (subscription, uri);
 		g_free (uri);
 	}
 	
