@@ -59,11 +59,12 @@ static struct enclosureDownloadTool enclosure_download_commands[] = {
 	{ "wget -q -O %s %s", TRUE },
 	{ "curl -s -o %s %s", TRUE },
 	{ "dbus-send --session --dest=org.gnome.gwget.ApplicationService /org/gnome/gwget/Gwget org.gnome.gwget.Application.OpenURI string:%s uint32:0", FALSE },
+	{ "kget %s", FALSE },
 	NULL
 };
 
 /** order must match enclosure_download_commands[] */
-static gchar *enclosure_download_tool_options[] = { "wget", "curl", "gwget", NULL };
+static gchar *enclosure_download_tool_options[] = { "wget", "curl", "gwget", "kget", NULL };
 
 static struct browser browsers[] = {
 	{
@@ -252,8 +253,10 @@ on_folderhidereadbtn_toggled (GtkToggleButton *togglebutton, gpointer user_data)
 	gboolean enabled = gtk_toggle_button_get_active (togglebutton);
 	conf_set_bool_value (FOLDER_DISPLAY_HIDE_READ, enabled);
 	displayedNode = itemlist_get_displayed_node ();
-	if (displayedNode)
+	if (displayedNode) {
+		itemlist_unload (FALSE);
 		itemlist_load (displayedNode);
+	}
 }
 
 void
@@ -557,7 +560,7 @@ ui_prefs_setup_combo_menu (const gchar *widgetName,
 	g_assert (NULL != widget);
 	for (i = 0; options[i] != NULL; i++) {
 		gtk_list_store_append (listStore, &treeiter);
-		gtk_list_store_set (listStore, &treeiter, 0, options[i], 1, i, -1);
+		gtk_list_store_set (listStore, &treeiter, 0, _(options[i]), 1, i, -1);
 	}
 	gtk_combo_box_set_model (GTK_COMBO_BOX (widget), GTK_TREE_MODEL (listStore));
 	if (-1 <= defaultValue)
